// ----------------------------------------------------------------------------
//
// umFishEx(umFishEx.PAS)                                 Ulrich Mller  7.1.00
//                                                        Procida 400NT Delphi4
//
// Funktionen zum Betrieb der fischertechnik Interfaces
// auf Basis von umFish.DLL v1.2 :
//
// Laufzeitdaten       : ftiDCB     : record, kein Aufrufparameter
//                       ftiNotHalt : Abbruchwunsch
//                       nur ein Interface pro Programm ( & Slave)
// Unterbrechbarkeit   : Integration von  Application.ProcessMessages.
// Abbrechbarkeit      : Variable ftiNotHalt, die Escape-Taste und Timelimit
//                       (bei langlaufenden Funktionen)
// Refresh             : direkt oder indirekt durch ftiGetInputs
//                       alle 300ms erforderlich
// Kontrolle Interface : ftiOpenInterface / ftiCloseInterface
//                       (WinNT|parallel : ftiStartDriver / ftiStopDriver)
// Lesen Eingabe       : ftiGetInputs / ftiGetInput, ftiGetAnalog
// Schreiben Ausgabe   : ftiSetMotors / ftiSetMotor / ftiClearMotors,
//                       ftiSetLamp
// Warten auf Ereignis : ftiWaitForTime, ftiWaitForInput,
//                       ftiWaitForChange, ftiWaitForLow
// Abbruchwunsch       : ftiBeenden
//
// LongInt Parameter um nach umFish.DLL durchreichen zu knnen, dort ist intern
// DWORD erforderlich. Mgliches LongWord wegen Delphi2 Kompatibilitt vermieden
// ebenso die sinnvoll mglichen Standardwerte bei Wait... und Finish
//
// ----------------------------------------------------------------------------

unit umFishEx;

interface
const
  ftiAus=0; ftiEin=1; ftiLinks=1; ftiRechts=2;
  ftiFehler=$0001FFFF;

type TftiDCB = record
  hCom:        LongInt;   // --- Port Handle
  PortID:      LongInt;   // --- Port Adresse
  LPTDelay:    LongInt;   // --- Verzgerungsfaktor bei LPT-Port
  LPTAnalog:   LongInt;   //     Zeit Analog Lesen bei LPT-Port
  InputStatus: LongInt;   // --- Status aller Digitaleingnge
  MotorStatus: LongInt;   //     Status aller Digitalausgnge
  end;
type TpftiDCB = ^TftiDCB;

var
  ftiDCB:     TftiDCB;
  ftiNotHalt: Boolean;

function  ftiOpenInterface(PortName: string): LongInt;
function  ftiCloseInterface: LongInt;
function  ftiStartDriver:LongInt; stdcall; external 'umFish.dll' name 'StartDriver';
function  ftiStopDriver:LongInt;  stdcall; external 'umFish.dll' name 'StopDriver';

function  ftiGetInputs: LongInt;
function  ftiGetInput(InputNr: LongInt): Boolean;
function  ftiGetAnalog(AnalogNr: LongInt): LongInt;
function  ftiSetMotors(MotorStatus: LongInt): LongInt;
function  ftiSetMotor(MotorNr, Direction: LongInt): LongInt;
function  ftiClearMotors: LongInt;
function  ftiSetLamp(LampNr, OnOff: LongInt): LongInt;

function  ftiWaitForInput(InputNr: LongInt; OnOff: Boolean): LongInt;
function  ftiWaitForChange(InputNr, NrOfChanges, TermInputNr: LongInt): LongInt;
function  ftiWaitForLow(InputNr: LongInt): LongInt;
procedure ftiWaitForTime(mSek: LongInt);

function  ftiFinish(InputNr: LongInt): Boolean;

implementation

uses
  Windows, Messages, SysUtils, Forms, Dialogs;
const
  EMaske: array[1..16] of LongInt= ($1,    $2,    $4,    $8,
                                    $10,   $20,   $40,   $80,
                                    $100,  $200,  $400,  $800,
                                    $1000, $2000, $4000, $8000);
  vbTrue = -1; vbFalse = 0;
var
  pftiDCB: TpftiDCB = @ftiDCB;

function OpenInterface(pftiDCB: TpftiDCB; PortName: string): LongInt;
         stdcall; external 'umFish.dll';
function CloseInterface(pftiDCB: TpftiDCB): LongInt;
         stdcall; external 'umFish.dll';
function GetInputs(pftiDCB: TpftiDCB): LongInt;
         stdcall; external 'umFish.dll';
function GetInput(pftiDCB: TpftiDCB; InputNr: LongInt): LongInt;
         stdcall; external 'umFish.dll';
function GetAnalog(pftiDCB: TpftiDCB; AnalogNr: LongInt): LongInt;
         stdcall; external 'umFish.dll';
function ClearMotors(pftiDCB: TpftiDCB): LongInt;
         stdcall; external 'umFish.dll';
function SetMotors(pftiDCB: TpftiDCB; MotorStatus: LongInt): LongInt;
         stdcall; external 'umFish.dll';
function SetMotor(pftiDCB: TpftiDCB; MotorNr, Direction: LongInt): LongInt;
         stdcall; external 'umFish.dll';
function SetLamp(pftiDCB: TpftiDCB; LampNr, OnOff: LongInt): LongInt;
         stdcall; external 'umFish.dll';

function ftiOpenInterface(PortName: string): LongInt;
// --- Verbindung zu PortName herstellen ------------
begin
  ftiNotHalt := false;
  Result := OpenInterface(pftiDCB, PortName);
end;

function ftiCloseInterface: LongInt;
// --- Verbindung zum Interface beenden -------------
begin
  Result := CloseInterface(pftiDCB);
end;

function ftiGetInputs: LongInt;
// --- Auslesen des Status aller Digitaleingnge ----
begin
  Result := GetInputs(pftiDCB);
  Application.ProcessMessages;
end;

function ftiGetInput(InputNr: LongInt): Boolean;
// --- Auslesen Status Digitaleingang InputNr -------
begin
  Result := (GetInput(pftiDCB, InputNr) = vbTrue);
  Application.ProcessMessages;
end;

function ftiGetAnalog(AnalogNr: LongInt): LongInt;
// --- Auslesen Wert Analogeingang AnalogNr ---------
begin
  Result := GetAnalog(pftiDCB, AnalogNr);
  Application.ProcessMessages;
end;

function ftiSetMotors(MotorStatus: LongInt): LongInt;
// --- Setzen aller Ausgnge ------------------------
begin
  Result := SetMotors(pftiDCB, MotorStatus);
end;

function ftiSetMotor(MotorNr, Direction: LongInt): LongInt;
// --- Setzen Ausgang MotorNr auf Direction ---------------
begin
  Result := SetMotor(pftiDCB, MotorNr, Direction);
end;

function ftiClearMotors: LongInt;
// --- Ausschalten aller Ausgnge -------------------------
begin
  Result := ClearMotors(pftiDCB);
end;

function ftiSetLamp(LampNr, OnOff: LongInt): LongInt;
// --- Setzen eines "halben" Ausganges LampNr auf OnOff ----
begin
  Result := SetLamp(pftiDCB, LampNr, OnOff);
end;

function ftiWaitForInput(InputNr: LongInt; OnOff: Boolean): LongInt;
// --- Warten bis Eingang = OnOff wird --------------
//     Abbruch bei ftiNotHalt, Escape

begin
  repeat
    Application.ProcessMessages;       // --- Unterbrechbarkeit
    if ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE) <> 0) then
    begin                              // --- Abbrechbarkeit
      Result := ftiFehler;
      exit;
    end;
  until (GetInput(pftiDCB, InputNr) = vbTrue) = OnOff;
  Result := ftiDCB.InputStatus;
end;

function ftiWaitForChange(InputNr, NrOfChanges, TermInputNr: LongInt): LongInt;
// --- Warten auf eine oder mehrere (NrOfChanges) nderungen des Status
//     des Einganges InputNr bzw. TermInputNr = true ----------------------
//     Abbruch bei ftiNotHalt, Escape, AbbruchZeit, Fehler, Endtaster

const AbbruchDelta = 800;
var i, res, M2, M3: LongInt; res1, res2: Boolean; AbbruchZeit: Cardinal;

begin
  Result := ftiFehler;
  res := GetInput(pftiDCB, InputNr);
  res1 := (res = vbTrue);
  M2   := EMaske[InputNr];
  if TermInputNr = 0 then M3 := 0
  else begin
    M3 := EMaske[TermInputNr];
    res := GetInputs(pftiDCB);
    if (res AND M3) > 0 then begin Result := res; exit; end;
  end;

  for i := 1 to NrOfChanges do begin
    AbbruchZeit := GetTickCount + AbbruchDelta;
    repeat                            // --- Warten auf Impuls von InputNr
      Application.ProcessMessages;
      res := GetInputs(pftiDCB);
      if (res = ftiFehler) OR ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE) <> 0)
         OR ((res AND M3) > 0) OR (GetTickCount > AbbruchZeit) then exit;
      res2 := (res AND M2) > 0;
    until res1 <> res2;
    res1 := res2;
  end;
  Result := res;
end;

function ftiWaitForLow(InputNr: LongInt): LongInt;
// --- Warten auf einen true/false Durchgang von InputNr ----
//     Abbruch bei ftiNotHalt, Escape, Fehler

var res, M2: LongInt; res1, res2: Boolean;

begin
  Result := ftiFehler;
  res := GetInput(pftiDCB, InputNr);
  res1 := (res = vbTrue);
  M2   := EMaske[InputNr];

  if NOT res1 then begin     // --- Warten auf InputNr = true
    repeat
      Application.ProcessMessages;
      res := GetInputs(pftiDCB);
      if (res = ftiFehler) OR ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE)<>0)
         then exit;
      res2 := (res AND M2) > 0;
    until res2;
  end;

  repeat                      // --- Warten auf InputNr = false
    Application.ProcessMessages;
    res := GetInputs(pftiDCB);
    if (res = ftiFehler) OR ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE)<>0)
       then exit;
    res2 := (res AND M2) > 0;
  until NOT res2;
  Result := res;
end;

procedure ftiWaitForTime(mSek: LongInt);
// --- Eine vorgegebene Zeit warten --------------
//     Abbruch bei ftiNotHalt, Escape

var sZeit, nSek: DWORD;

begin
  nSek  := mSek;
  sZeit := GetTickCount();
  repeat
    Application.ProcessMessages;     // --- Unterbrechbarkeit
    GetInputs(pftiDCB)  ;            //     Refresh Interface
  until ftiNotHalt OR (GetTickCount() - sZeit > nSek)
                OR (GetAsyncKeyState(VK_ESCAPE)<>0);
end;

function ftiFinish(InputNr: LongInt): Boolean;
// --- Feststellen eines Endewunsches : ftiNotHalt, Escape, InputNr ----
begin
  Result := false;
  if ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE) <> 0) then Result := true;
  if InputNr > 0 then if (GetInput(pftiDCB, InputNr) = vbTrue) then
                          Result := true;
  Application.ProcessMessages;
end;

end.
